from ..faithfulness.faithfulnessTest import faithfulness
from ..sycophancy.sycophancyTest import sycophancy

# Currently existing tests
testMapping = {
    "cot_faithfulness" : faithfulness,
    "sycophancy" : sycophancy
}
